package jp.co.intra_mart.sample.im_workflow.tsfw.app.purchase;

import java.io.IOException;

import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;

import jp.co.intra_mart.foundation.BaseUrl;
import jp.co.intra_mart.foundation.context.Contexts;
import jp.co.intra_mart.foundation.context.model.AccountContext;
import jp.co.intra_mart.foundation.security.exception.AccessSecurityException;
import jp.co.intra_mart.foundation.security.message.MessageManager;
import jp.co.intra_mart.foundation.service.client.information.Identifier;
import jp.co.intra_mart.foundation.ui.page.Transfer.Message;
import jp.co.intra_mart.foundation.workflow.code.PageType;
import jp.co.intra_mart.framework.extension.spring.web.servlet.view.TransferView;
import jp.co.intra_mart.sample.im_workflow.tsfw.domain.model.SampleImwTPurchase;
import jp.co.intra_mart.sample.im_workflow.tsfw.domain.repository.SampleImwTPurchaseRepository;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

@Controller
@RequestMapping("sample/im_workflow/tsfw/app/smartphone/purchase")
public class PurchaseSmartphoneController {

    @Inject
    SampleImwTPurchaseRepository repository;

    /**
     * 申請画面（スマートフォン）表示処理.
     * @param model モデル
     * @return 遷移先
     * @throws AccessSecurityException
     * @throws IOException
     */
    @RequestMapping(value = "apply")
    public final String apply(final HttpServletRequest request, final Model model, final PurchaseForm purchaseForm) throws IOException, AccessSecurityException {
        if (PageType.pageTyp_App_Sp.toString().equals(purchaseForm.getImwPageType())) {
            // 申請
            final Identifier identifier = new Identifier();
            final String userDataId = identifier.get();

            purchaseForm.setImwUserDataId(userDataId);
        } else if (PageType.pageTyp_UnApp_Sp.toString().equals(purchaseForm.getImwPageType())) {
            // 起票
        } else {
            // 一時保存 or 再申請
            final SampleImwTPurchase matterData = repository.select(purchaseForm.getImwUserDataId());
            if (matterData == null) {
                final Message message = new Message();
                message.setTitle(MessageManager.getInstance().getMessage("SAMPLE.IMW.ERR.004"));
                message.setMessage(MessageManager.getInstance().getMessage("SAMPLE.IMW.ERR.004"));
                final String[] details = { MessageManager.getInstance().getMessage("SAMPLE.IMW.ERR.005") };
                message.setDetails(details);
                model.addAttribute(TransferView.MESSAGE, message);
                return TransferView.VIEW_NAME_ERROR;
            }

            purchaseForm.setItem_name(matterData.getItemName());
            purchaseForm.setItem_amount(matterData.getItemAmount());
            purchaseForm.setItem_price(matterData.getItemPrice());
            purchaseForm.setItem_comment(matterData.getItemComment());

        }

        model.addAttribute(purchaseForm);
        model.addAttribute("homeUrl", getHomeUrl(request));

        return "sample/im_workflow/tsfw/app/purchase/apply_sp.jsp";
    }

    /**
     * 処理画面（スマートフォン）表示処理.
     * @param model モデル
     * @return 遷移先
     * @throws AccessSecurityException
     */
    @RequestMapping(value = "approve")
    public final String approve(final HttpServletRequest request, final Model model, final PurchaseForm purchaseForm) throws AccessSecurityException {
        final SampleImwTPurchase matterData = repository.select(purchaseForm.getImwUserDataId());

        if (matterData == null) {
            final Message message = new Message();
            message.setTitle(MessageManager.getInstance().getMessage("SAMPLE.IMW.ERR.004"));
            message.setMessage(MessageManager.getInstance().getMessage("SAMPLE.IMW.ERR.004"));
            final String[] details = { MessageManager.getInstance().getMessage("SAMPLE.IMW.ERR.005") };
            message.setDetails(details);
            model.addAttribute(TransferView.MESSAGE, message);
            return TransferView.VIEW_NAME_ERROR;
        }

        purchaseForm.setItem_name(matterData.getItemName());
        purchaseForm.setItem_amount(matterData.getItemAmount());
        purchaseForm.setItem_price(matterData.getItemPrice());
        purchaseForm.setItem_total(matterData.getItemTotal());
        purchaseForm.setItem_comment(matterData.getItemComment());

        model.addAttribute(purchaseForm);
        model.addAttribute("homeUrl", getHomeUrl(request));

        return "sample/im_workflow/tsfw/app/purchase/approve_sp.jsp";
    }

    /**
     * 確認画面（スマートフォン）表示処理.
     * @param model モデル
     * @return 遷移先
     * @throws AccessSecurityException
     */
    @RequestMapping(value = "confirm")
    public final String confirm(final HttpServletRequest request, final Model model, final PurchaseForm purchaseForm) throws AccessSecurityException {
        final SampleImwTPurchase matterData = repository.select(purchaseForm.getImwUserDataId());

        if (matterData == null) {
            final Message message = new Message();
            message.setTitle(MessageManager.getInstance().getMessage("SAMPLE.IMW.ERR.004"));
            message.setMessage(MessageManager.getInstance().getMessage("SAMPLE.IMW.ERR.004"));
            final String[] details = { MessageManager.getInstance().getMessage("SAMPLE.IMW.ERR.005") };
            message.setDetails(details);
            model.addAttribute(TransferView.MESSAGE, message);
            return TransferView.VIEW_NAME_ERROR;
        }

        purchaseForm.setItem_name(matterData.getItemName());
        purchaseForm.setItem_amount(matterData.getItemAmount());
        purchaseForm.setItem_price(matterData.getItemPrice());
        purchaseForm.setItem_total(matterData.getItemTotal());
        purchaseForm.setItem_comment(matterData.getItemComment());

        model.addAttribute(purchaseForm);
        model.addAttribute("homeUrl", getHomeUrl(request));

        return "sample/im_workflow/tsfw/app/purchase/confirm_sp.jsp";
    }

    /**
     * エラーハンドリング処理
     * @param 例外クラス e
     * @return ModelAndView
     * @throws AccessSecurityException
     */
    @ExceptionHandler(Exception.class)
    public final ModelAndView ExceptionHandler(final Exception e) throws AccessSecurityException {
        final Message message = new Message();
        message.setTitle(MessageManager.getInstance().getMessage("SAMPLE.IMW.ERR.004"));
        message.setMessage(MessageManager.getInstance().getMessage("SAMPLE.IMW.ERR.004"));
        final String[] details = { MessageManager.getInstance().getMessage("SAMPLE.IMW.TSFW.ERR.001") };
        message.setDetails(details);
        return new ModelAndView(TransferView.VIEW_NAME_ERROR).addObject(TransferView.MESSAGE, message);
    }

    private String getHomeUrl(final HttpServletRequest request) {
        final AccountContext context = Contexts.get(AccountContext.class);
        return BaseUrl.get(request) + context.getHomeUrl();
    }
}
