package jp.co.intra_mart.sample.im_workflow.tsfw.domain.job;

import java.util.HashMap;
import java.util.Map;

import javax.inject.Inject;

import jp.co.intra_mart.foundation.job_scheduler.JobResult;
import jp.co.intra_mart.foundation.workflow.application.model.ApplyResultModel;
import jp.co.intra_mart.foundation.workflow.application.model.param.ApplyParam;
import jp.co.intra_mart.foundation.workflow.application.process.ApplyManager;
import jp.co.intra_mart.foundation.workflow.exception.WorkflowException;
import jp.co.intra_mart.sample.im_workflow.tsfw.domain.model.SampleImwTPurchase;
import jp.co.intra_mart.sample.im_workflow.tsfw.domain.repository.SampleImwTPurchaseRepository;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class SampleApplyJobService {

    @Inject
    SampleImwTPurchaseRepository repository;

    public JobResult execute() throws WorkflowException {
        // ワークフローデータの作成
        final ApplyParam param = new ApplyParam();
        param.setApplyAuthUserCode("ueda");
        param.setApplyExecuteUserCode("ueda");
        param.setFlowId("flow_tsfw");
        param.setApplyBaseDate("2015/10/07");
        param.setMatterName("Sample_Matter");
        // ユーザプリケーションデータの作成
        final Map<String, Object> userParameter = new HashMap<String, Object>();
        userParameter.put("item_name", "sample_item");
        userParameter.put("item_amount", "1");
        userParameter.put("item_price", "1000");
        userParameter.put("item_comment", "sample_comment");

        // 申請処理の実行
        final ApplyManager manager = new ApplyManager();
        final ApplyResultModel result = manager.apply(param, userParameter);

        // 申請時に登録されたユーザプリケーションデータをコンソールに出力します。
        SampleImwTPurchase entity = repository.select(result.getUserDataId());
        outputLog(entity);

        // ユーザプリケーションデータを更新します。
        update(entity);

        // 更新されたユーザプリケーションデータをコンソールに出力します。
        entity = repository.select(result.getUserDataId());
        outputLog(entity);

        return JobResult.success("success");
    }

    private void outputLog(final SampleImwTPurchase entity) {
        System.out.println("----- SampleApplyJob Result -----");
        System.out.println("item_name :      " + entity.getItemName());
        System.out.println("item_amount :    " + entity.getItemAmount());
        System.out.println("item_price :     " + entity.getItemPrice());
        System.out.println("item_comment :   " + entity.getItemComment());
        System.out.println("----- SampleApplyJob Result -----");
        System.out.println("");
    }

    @Transactional(rollbackFor = java.lang.Exception.class)
    private void update(final SampleImwTPurchase entity) {
        entity.setItemComment("sample_comment update");
        repository.update(entity);
    }

}
