package jp.co.intra_mart.sample.im_workflow.tsfw.initialize;

import java.io.IOException;

import jp.co.intra_mart.foundation.context.Contexts;
import jp.co.intra_mart.foundation.context.model.AccountContext;
import jp.co.intra_mart.foundation.security.ExtendsImport;
import jp.co.intra_mart.foundation.security.exception.AccessSecurityException;
import jp.co.intra_mart.foundation.service.client.file.PublicStorage;
import jp.co.intra_mart.foundation.service.client.file.SystemStorage;
import jp.co.intra_mart.foundation.workflow.exception.WorkflowException;
import jp.co.intra_mart.system.workflow.util.WorkflowImportUtil;
import jp.co.intra_mart.system.workflow.util.model.ImportDataListModel;

/**
 * 拡張インポート処理（サンプルフローのインポート）
 * @author INTRAMART
 */
public class SampleFlowImporter implements ExtendsImport {

    private static final String IMPORT_DIR = "import_export";

    private static final String SEPARATOR = "/";

    private static final String SYSTEM_ROOT = "products/import/basic/im_workflow_tsfw_sample/import";

    private static final String SAVE_ROOT = "im_workflow/data";

    private static final String WORKFLOW_FILE_NAME = "im_workflow_tsfw.xml";

    @Override
    public void doImport(final String groupId) throws AccessSecurityException {
        try {
            final String localeId = Contexts.get(AccountContext.class).getLocale().toString();
            final WorkflowImportUtil importUtil = new WorkflowImportUtil(groupId);
            doCopyImportFile(groupId);
            // 一時領域の作成
            final ImportDataListModel[] importArray = importUtil.inputXMLFile(localeId, WORKFLOW_FILE_NAME);
            // インポート処理
            for (final ImportDataListModel element : importArray) {
                if ("contents".equals(element.getDataType())) {
                    importUtil.importContentsData(localeId, element.getFileName());
                } else if ("route".equals(element.getDataType())) {
                    importUtil.importRouteData(localeId, element.getFileName());
                } else if ("flow".equals(element.getDataType())) {
                    importUtil.importFlowData(localeId, element.getFileName());
                }
            }
            // 一時領域の削除
            importUtil.removeTemporary();
        } catch (final WorkflowException e) {
            throw new AccessSecurityException(e.getMessage(), e);
        }
    }

    private void doCopyImportFile(final String groupId) throws AccessSecurityException {
        try {
            final SystemStorage originalFile = new SystemStorage(SYSTEM_ROOT + SEPARATOR + WORKFLOW_FILE_NAME);
            final PublicStorage copyFile = new PublicStorage(SAVE_ROOT + SEPARATOR + groupId + SEPARATOR + IMPORT_DIR + SEPARATOR + WORKFLOW_FILE_NAME);

            final PublicStorage copyDir = new PublicStorage(SAVE_ROOT + SEPARATOR + groupId + SEPARATOR + IMPORT_DIR);
            if (!copyDir.isDirectory()) {
                copyDir.makeDirectories();
            }

            if (originalFile.isFile()) {
                copyFile.save(originalFile.load());
            }
        } catch (final IOException e) {
            throw new AccessSecurityException(e.getMessage(), e);
        }
    }
}
